@@ -15,7 +15,7 @@ then
     exit 0
 fi
 
-if ! command -v restorecon &> /dev/null
+if ! command -v restorecon >/dev/null 2>&1
 then
     # restorecon is not available
     exit 0
