@@ -151,6 +151,9 @@ class DSLdapObject(DSLogging, DSLint):
     def __str__(self):
         return self.__unicode__()
 
+    def __repr__(self):
+        return f'{type(self)}(instance_name="{self._instance.serverid}", dn="{self.__unicode__()}")'
+
     def _unsafe_raw_entry(self):
         """Get an Entry object
 
@@ -313,6 +316,29 @@ class DSLdapObject(DSLogging, DSLint):
 
         self.set(key, value, action=ldap.MOD_ADD)
 
+    def add_many(self, *args):
+        """Add many key, value pairs in a single operation.
+        This is useful for configuration changes that require
+        atomic operation, and ease of use.
+
+        An example of usage is add_many((key, value), (key, value))
+
+        No wrapping list is needed for the arguments.
+
+        :param *args: tuples of key,value to replace.
+        :type *args: (str, str)
+        """
+
+        mods = []
+        for arg in args:
+            if isinstance(arg[1], list) or isinstance(arg[1], tuple):
+                value = ensure_list_bytes(arg[1])
+            else:
+                value = [ensure_bytes(arg[1])]
+            mods.append((ldap.MOD_ADD, ensure_str(arg[0]), value))
+        return _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls,
+                                            clientctrls=self._client_controls, escapehatch='i am sure')
+
     # Basically what it means;
     def replace(self, key, value):
         """Replace an attribute with a value
@@ -748,7 +774,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of bytes values
         :raises: ValueError - if instance is offline
         """
 
@@ -759,7 +785,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single UTF8 value
         :raises: ValueError - if instance is offline
         """
 
@@ -770,7 +796,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single lowered UTF8 value
         :raises: ValueError - if instance is offline
         """
 
@@ -785,7 +811,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of UTF8 values
         :raises: ValueError - if instance is offline
         """
 
@@ -796,7 +822,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of lowered UTF8 values
         :raises: ValueError - if instance is offline
         """
 
@@ -807,7 +833,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A single int value
         :raises: ValueError - if instance is offline
         """
 
@@ -818,7 +844,7 @@ class DSLdapObject(DSLogging, DSLint):
 
         :param key: An attribute name
         :type key: str
-        :returns: A single bytes value
+        :returns: A list of int values
         :raises: ValueError - if instance is offline
         """
 
